[![Circle CI](https://circleci.com/gh/wellington/wellington/tree/master.svg?style=svg)](https://circleci.com/gh/wellington/wellington/tree/master)
[![Coverage Status](https://coveralls.io/repos/wellington/wellington/badge.png?branch=master)](https://coveralls.io/r/wellington/wellington?branch=master)

Wellington
===========

Wellington adds the missing piece to SASS, file awareness.  Wellington can create relative paths to your assets and sprites from images in the image directory.  This tool is designed to work directly from sass, no need learning a new language or messing with separate spriting tools!

### [Try the Demo!](http://codepen.io/drewwells/details/KwggLx/)

```
$images: sprite-map("sprites/*.png");
div {
  width: image-width(sprite-file($images, "cat"));
  height: image-height(sprite-file($images, "cat"));
  background: sprite($images, "cat");
}
```
// Generates
```
div {
  width: 140px;
  height: 79px;
  background: url("genimg/sprites-wehqi.png");
}
```

### Why?

Sass makes CSS more awesome, Wellington makes Sass awesomer. Wellington extends your Sass to do things not supported in Sass.  Wellington can manage your web project rather than just your Sass files.  With Wellington, spriting and asset paths are now available to you directly in your Sass code.  Check out the [demo](http://codepen.io/drewwells/details/KwggLx/) or see the [examples](#examples).

Wellington has been written in Go for portability, modularity, and speed.  There are no dependencies on Ruby.  The binary includes everything you need to run sprite_sass.  Sprite_sass uses libsass under the covers for processing the output CSS.

#### Installation
Wellington can be installed via brew

	brew update
	brew tap drewwells/wt
	brew install wellington

Or, run it in docker

	docker run -v $(pwd):/data -it drewwells/wellington wt proj.scss

#### Building from source
Install Go and add $GOPATH/bin to your $PATH. [Detailed instructions](https://golang.org/doc/install)

You must have libsass installed to build this project.  Do so by checking
out the repo or building [libsass](https://github.com/sass/libsass/wiki/Building-on-Mac-OS-X) via the  instruction in the repo.

```
# This will fail if you don't have libsass installed, that's OK.
go get -u github.com/wellington/wellington/wt
cd $GOPATH/wellington/wellington
make deps
# Attempt install again
go get -u github.com/wellington/wellington/wt
```

Test out if the installation worked
```
wt
```

## Examples

### Mixins

#### @include sprite-dimensions($map, "file")

Sprite-dimensions outputs the height and width properties of the specified image.

```
div {
  @include sprite-dimensions($spritemap, "file");
}
```

*Output*

```css
div {
	width: 100px;
	height: 50px;
}
```

### Functions

Don't see a function you want?  Check out [handlers](http://godoc.org/github.com/wellington/wellington/context/handlers) and submit a pull request!

#### sprite-map("glob/pattern"[, $spacing: 10px])

sprite-map generates a sprite from the matched images with the specified spacing between them.  No output is generated by this function, instead the returned variable is sent to other functions.

```
$spritemap: sprite-map("*.png");
```

*Output*

```css

```

#### sprite($map, $name: "image"[, $offsetX: 0px, $offsetY: 0px])|

sprite generates a background url with background position to the position of the specified `"image"` in the spritesheet.  Optionally, offsets can be used to slightly modify the background position.

```
div {
	background: sprite($spritemap, "image");
}
```

*Output*

```css
div {
	background: url("spritegen.png") -0px -149px;
}
```

#### sprite-file($map, $name: "image")

Sprite-file returns an encoded string only useful for passing to image-width or image-height.

```
div {
	background: sprite-file($spritemap, "image");
}
```

*Output*

```css
div {
	background: {encodedstring};
}
```

#### image-height($path)

image-height returns the height of the image specified by `$path`.

```
div {
	height: image-height(sprite-file($spritemap, "image"));
}
div {
	height: image-height("path/to/image.png");
}
```

*Output*

```css
div {
	height: 50px;
}
div {
	height: 50px;
}
```

#### image-width($path)

image-width returns the width of the image specified by `$path`.

```
div {
	width: image-width(sprite-file($spritemap, "image"));
}
div {
	width: image-width("path/to/image.png");
}
```

*Output*

```css
div {
	width: 50px;
}
div {
	width: 50px;
}
```

#### inline-image($path[, $encode: false])

inline-image creates a base64 encoded png. When the image is svg, it returns an inline svg that can optionally be base64 encoded.  Encoding incurs a file size penalty.

```
div {
	background: inline-image("path/to/image.png");
}
div.svg {
	background: inline-image("path/to/image.svg", $encode: false);
}
```

*Output*

```css
div {
	background: inline-image("data:image/png;base64,iVBOR...");
}
div.svg {
	background: inline-image("data:image/svg+xml;utf8,%3C%3F...");
}
```

#### image-url($path)

image-url returns a relative path to an image in the image directory from the built css directory.

```
div {
	background: image-url("path/to/image.png");
}
```

*Output*

```css
div {
	background: url('../imgdirectory/path/to/image.png");
}
```

### font-url($path, [$raw:false])

font-url returns a relative path to fonts in your font directory.  You must set the font path to use this function.  By default, font-url will return `url('path/to/font')`, set `$raw: true` to only return the path

```
div {
	$path: font-url("arial.eot", true);
	@font-face {
		src: font-url("arial.eot");
		src: url("#{$path}");
	}
}
```

*Output*

```css
div {
	@font-face {
		src: url("../font/arial.eot");
		src: url("../font/arial.eot");
	}
}
```

### Development

Get the code

	go get github.com/wellington/wellington

You may want to cd `$GOPATH/wellington/wellington` and set the origin to your fork.

	git remote rm origin
	git remote add origin git@github.com:username/wellington.git

Testing

    make test

Profiling

	make profile

Docker Container

	make build
	make docker #launch a container

Please use pull requests for contributing code.  Wercker will automatically test and lint your contributions.  Thanks for helping!

### License

Wellington is licensed under MIT.
